%
%   INVPNNL2.M   Nonlinear model of inverted pendulum (Simulink S-function format)
%
%   See documentation of equations in Section VII of Dynamic Systems Notes
%   The interpretation of the flag variable is obtained by typing: help sfuntmpl 
%
%   File prepared by J. R. White, UMass-Lowell (last update: March 2020).
%

      function [zdot,zo] = invpnnl2(t,z,u,flag,M,m,g,len)
%
%   return parameter size and initial conditions
      if flag == 0,    zdot = [4 0 2 1 0 0];    zo = zeros(4,1); 
%
%   return continuous state derivatives (as a column vector)
      elseif flag == 1
        c1 = (M+m);  c2 = m*len;  c3 = m*g;  c4 = (M+m)*len;  c5 = (M+m)*g;
        zdot(1) = z(2);
        top2 = u*cos(z(1)) - c5*sin(z(1)) + c2*cos(z(1))*sin(z(1))*z(2)^2;
        zdot(2) = top2/(c2*cos(z(1))^2 - c4);
        zdot(3) = z(4);
        top4 = u + c2*sin(z(1))*z(2)^2 - c3*cos(z(1))*sin(z(1));
        zdot(4) = top4/(c1-m*cos(z(1))^2);
        zdot = zdot';
%
%   return the output vector (rod and cart position)  (as a column vector)
      elseif flag == 3
        zdot(1) = z(1);   zdot(2) = z(3);   zdot = zdot';
%
%   other options not used
      else 
        zdot = [];  
      end
% 
%   end of routine
