%
%   INVPNNL3.M   Nonlinear model of closed loop (state controlled) inverted pendulum 
%             
%   File prepared by J. R. White, UMass-Lowell (last update: March 2020)
%

      function Zdot = invpnnl3(t,Z,M,m,g,len,A,B,C,D,Ks,Nr,L,rd)
%
%   separate plant and estimator states
      nn = max(size(A));
      zp = Z(1:nn,1);     ze = Z(nn+1:2*nn,1);
%   now calc derivative of full state vector (plant states and estimator states)
      yp = C*zp;   u = Nr*rd - Ks*ze;
      c1 = (M+m);  c2 = m*len;  c3 = m*g;  c4 = (M+m)*len;  c5 = (M+m)*g;
%   plant states
      zpdot(1) = zp(2);
      top2 = u*cos(zp(1)) - c5*sin(zp(1)) + c2*cos(zp(1))*sin(zp(1))*zp(2)^2;
      zpdot(2) = top2/(c2*cos(zp(1))^2 - c4);
      zpdot(3) = zp(4);
      top4 = u + c2*sin(zp(1))*zp(2)^2 - c3*cos(zp(1))*sin(zp(1));
      zpdot(4) = top4/(c1-m*cos(zp(1))^2);
%   estimator states
      zedot = A*ze + L*(yp - C*ze) + B*u;
%   put full state vector back together
      Zdot = [zpdot'; zedot];
%
%  end of routine
